/**
 * @license
 * Copyright 2022 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
import '../../../ripple/ripple.js';
import '../../../focus/focus-ring.js';
import { LitElement, nothing, PropertyValues, TemplateResult } from 'lit';
/**
 * Supported roles for a list item.
 */
export type ListItemRole = 'listitem' | 'menuitem' | 'option' | 'link' | 'none';
interface ListItemSelf {
    active: boolean;
    disabled: boolean;
}
/**
 * The interface of an item that is compatible with md-list. An item that is
 * selectable and disablable.
 */
export type ListItem = ListItemSelf & HTMLElement;
export declare class ListItemEl extends LitElement implements ListItem {
    /**
     * The primary, headline text of the list item.
     */
    headline: string;
    /**
     * The one-line supporting text below the headline. Set
     * `multiLineSupportingText` to `true` to support multiple lines in the
     * supporting text.
     */
    supportingText: string;
    /**
     * Modifies `supportingText` to support multiple lines.
     */
    multiLineSupportingText: boolean;
    /**
     * The supporting text placed at the end of the item. Overridden by elements
     * slotted into the `end` slot.
     */
    trailingSupportingText: string;
    /**
     * Disables the item and makes it non-selectable and non-interactive.
     */
    disabled: boolean;
    /**
     * The tabindex of the underlying item.
     *
     * __NOTE:__ this is overridden by the keyboard behavior of `md-list` and by
     * setting `selected`.
     */
    itemTabIndex: number;
    /**
     * Whether or not the element is actively being interacted with by md-list.
     * When active, tabindex is set to 0, and in some list item variants (like
     * md-list-item), focuses the underlying item.
     */
    active: boolean;
    /**
     * Sets the role of the list item. Set to '' to clear the role.
     */
    type: ListItemRole;
    /**
     * READONLY. Sets the `md-list-item` attribute on the element.
     */
    isListItem: boolean;
    protected readonly listItemRoot: HTMLElement | null;
    /**
     * Only meant to be overridden by subclassing and not by the user. This is
     * so that we have control over focus on specific variants such as disabling
     * focus on <md-autocomplete-item> but enabling it for <md-menu-item>.
     */
    protected focusOnActivation: boolean;
    protected isFirstUpdate: boolean;
    protected willUpdate(changed: PropertyValues<this>): void;
    protected render(): TemplateResult<1>;
    /**
     * Renders the root list item.
     *
     * @param content the child content of the list item.
     */
    protected renderListItem(content: unknown): TemplateResult<1>;
    /**
     * Handles rendering of the ripple element.
     */
    protected renderRipple(): TemplateResult | typeof nothing;
    /**
     * Handles rendering of the focus ring.
     */
    protected renderFocusRing(): TemplateResult | typeof nothing;
    /**
     * Classes applied to the list item root.
     */
    protected getRenderClasses(): {
        'with-one-line': boolean;
        'with-two-line': boolean;
        'with-three-line': boolean;
        disabled: boolean;
    };
    /**
     * The content rendered at the start of the list item.
     */
    protected renderStart(): TemplateResult<1>;
    /**
     * Handles rendering the headline and supporting text.
     */
    protected renderBody(): TemplateResult<1>;
    /**
     * Renders the one-line supporting text.
     */
    protected renderSupportingText(): TemplateResult<1>;
    /**
     * Gets the classes for the supporting text node
     */
    protected getSupportingTextClasses(): {
        'supporting-text--multi-line': boolean;
    };
    /**
     * The content rendered at the end of the list item.
     */
    protected renderEnd(): TemplateResult<1>;
    /**
     * Renders the supporting text at the end of the list item.
     */
    protected renderTrailingSupportingText(): TemplateResult<1>;
    protected onClick?(event: Event): void;
    protected onKeydown?(event: KeyboardEvent): void;
    protected onPointerenter?(event: Event): void;
    protected onPointerleave?(event: Event): void;
    protected updated(changed: PropertyValues<this>): void;
    focus(): void;
}
export {};
